﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;

namespace ConsoleApplication1
{
    class Program
    {
        static void Main(string[] args)
        {
            Int32 records = 150;
            List<Person> persons = new List<Person>(records);
            for (Int32 i = 0; i < records; i++)
            {
                Person person = new Person();
                person.ID = i + 1;
                person.Name = "name" + (i + 1);
                person.Guid = Guid.NewGuid();
                person.Salary = Math.Abs(Guid.NewGuid().GetHashCode() % 10000) + 5000;
                person.Birth = DateTime.Now;
                persons.Add(person);
            }

            //List<Person> persons = new List<Person>(2);
            //persons.Add(new Person()
            //{
            //    ID = 1,
            //    Name = "Mike",
            //    Birth = DateTime.Now,
            //    Salary = 1234567890123456M
            //});
            //persons.Add(new Person()
            //{
            //    ID = 2,
            //    Name = "Rattz",
            //    Birth = DateTime.Now,
            //    Salary = 1234567890123456M
            //});

            List<ExcelHelper.Header> headers = new List<ExcelHelper.Header>();
            headers.Add(new ExcelHelper.Header("ID"));
            headers.Add(new ExcelHelper.Header("Name", "名称"));
            headers.Add(new ExcelHelper.Header("Birth", "生日", "yyyy-MM-dd"));
            headers.Add(new ExcelHelper.Header("Salary", "薪水"));
            //headers.Add(new ExcelHelper.Header("Salary", "薪水", "F0"));

            ExcelHelper excelHelper = new ExcelHelper();
            excelHelper.RowPerSheet = 100;
            String path = @"c:\1.xls";
            IWorkbook workbook = excelHelper.Export<Person>(persons, headers);
            using (FileStream stream = File.Open(path, FileMode.OpenOrCreate))
            {
                workbook.Write(stream);
            }

            //Person person = new Person();
            //person.Salary = 1234567890123456;
            //Console.WriteLine(person.Salary.ToString("P2"));
        }
    }

    public class ExcelHelper
    {
        public class Header
        {
            public String Name { get; set; }
            public String PrintName { get; set; }
            public String Format { get; set; }

            public Header(String name)
                : this(name, name)
            {
            }

            public Header(String name, String printName)
                : this(name, printName, null)
            {
            }

            public Header(String name, String printName, String format)
            {
                Name = name;
                PrintName = printName;
                Format = format;
            }
        }

        private const Int32 MaxRowPerSheet = 65535;
        private Int32 rowPerSheet = 1000;
        public Int32 RowPerSheet
        {
            get { return rowPerSheet; }
            set
            {
                if (value < 0 || value > MaxRowPerSheet)
                {
                    throw new ArgumentOutOfRangeException("RowPerSheet");
                }
                else
                {
                    rowPerSheet = value;
                }
            }
        }

        public IWorkbook Export<T>(IList<T> records)
        {
            if (records == null)
                throw new ArgumentNullException("records");

            String[] headers = typeof(T).GetProperties().Select(p => p.Name).ToArray();
            return Export<T>(records, headers);
        }

        public IWorkbook Export<T>(IList<T> records, IList<String> headers)
        {
            if (records == null)
                throw new ArgumentNullException("records");
            if (headers == null)
                throw new ArgumentNullException("headers"); //这里对headers集合非空、重复性、子元素非空引用等检查视具体需求

            Header[] newHeaders = typeof(T).GetProperties().Select(p => new Header(p.Name)).ToArray();
            return Export<T>(records, newHeaders);
        }

        public IWorkbook Export<T>(IList<T> records, IList<Header> headers)
        {
            if (records == null)
                throw new ArgumentNullException("records");
            if (headers == null)
                throw new ArgumentNullException("headers");

            PropertyInfo[] props = new PropertyInfo[headers.Count];
            for (Int32 i = 0; i < headers.Count; i++)
            {
                props[i] = typeof(T).GetProperty(headers[i].Name); //属性数组可能产生空元素
            }

            IWorkbook workbook = new HSSFWorkbook();
            ISheet sheet = null;
            IRow row = null;


            for (Int32 r = 0; r < records.Count; r++)
            {
                if ((r % RowPerSheet) == 0)
                {
                    Int32 sheetIndex = (Int32)((Double)r / RowPerSheet) + 1;
                    sheet = workbook.CreateSheet("Sheet" + sheetIndex); //工作表名，请按自己需求实现
                    row = sheet.CreateRow(0); //创建表头
                    for (Int32 i = 0; i < headers.Count; i++)
                    {
                        row.CreateCell(i).SetCellValue(headers[i].PrintName);
                    }
                }

                row = sheet.CreateRow((r % RowPerSheet) + 1); //因为表头存在，行标需要下移1位
                for (Int32 i = 0; i < props.Length; i++)
                {
                    if (props[i] != null) //空引用检查十分必要
                    {
                        Object value = props[i].GetValue(records[r], null);
                        if (value != null) //空引用检查十分必要
                        {
                            ICell cell = row.CreateCell(i);
                            if (props[i].PropertyType == typeof(Int32))
                            {
                                if (String.IsNullOrWhiteSpace(headers[i].Format))
                                {
                                    cell.SetCellValue(((Int32)value));
                                }
                                else
                                {
                                    cell.SetCellValue(((Int32)value).ToString(headers[i].Format));
                                }
                            }
                            else if (props[i].PropertyType == typeof(Double))
                            {
                                if (String.IsNullOrWhiteSpace(headers[i].Format))
                                {
                                    cell.SetCellValue(((Double)value));
                                }
                                else
                                {
                                    cell.SetCellValue(((Double)value).ToString(headers[i].Format));
                                }
                            }
                            else if (props[i].PropertyType == typeof(Decimal))
                            {
                                if (String.IsNullOrWhiteSpace(headers[i].Format))
                                {
                                    cell.SetCellValue(Convert.ToDouble(value));
                                }
                                else
                                {
                                    cell.SetCellValue(((Decimal)value).ToString(headers[i].Format));
                                }
                            }
                            else if (props[i].PropertyType == typeof(DateTime))
                            {
                                if (String.IsNullOrWhiteSpace(headers[i].Format))
                                {
                                    cell.SetCellValue((DateTime)value);
                                }
                                else
                                {
                                    cell.SetCellValue(((DateTime)value).ToString(headers[i].Format));
                                }
                            }
                            else
                            {
                                cell.SetCellValue(value.ToString());
                            }
                        }
                    }
                }
            }

            //调整每张Sheet列宽自适应
            for (Int32 i = 0; i < workbook.NumberOfSheets; i++)
            {
                sheet = workbook.GetSheetAt(i);
                for (Int32 h = 0; h < headers.Count; h++)
                {
                    sheet.AutoSizeColumn(h);
                }
            }
            return workbook;
        }
    }

    public class Person
    {
        public Int32 ID { get; set; }
        public Guid Guid { get; set; }
        public String Name { get; set; }
        public DateTime Birth { get; set; }
        public Double Salary { get; set; }
        //public Decimal Salary { get; set; }
    }
}
